import {
  DatabaseErrorException,
  NotFoundException,
} from '@mobicoop/ddd-library';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { DeleteAuthenticationGrpcController } from '@modules/authentication/interface/grpc-controllers/delete-authentication.grpc.controller';
import { DeleteAuthenticationRequestDto } from '@modules/authentication/interface/grpc-controllers/dtos/delete-authentication.request.dto';
import { CommandBus } from '@nestjs/cqrs';
import { RpcException } from '@nestjs/microservices';
import { Test, TestingModule } from '@nestjs/testing';

const deleteAuthenticationRequest: DeleteAuthenticationRequestDto = {
  userId: '78153e03-4861-4f58-a705-88526efee53b',
};

const mockCommandBus = {
  execute: jest
    .fn()
    .mockImplementationOnce(() => ({}))
    .mockImplementationOnce(() => {
      throw new NotFoundException();
    })
    .mockImplementationOnce(() => {
      throw new DatabaseErrorException();
    })
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

describe('Delete Authentication Grpc Controller', () => {
  let deleteAuthenticationGrpcController: DeleteAuthenticationGrpcController;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CommandBus,
          useValue: mockCommandBus,
        },
        DeleteAuthenticationGrpcController,
      ],
    }).compile();

    deleteAuthenticationGrpcController =
      module.get<DeleteAuthenticationGrpcController>(
        DeleteAuthenticationGrpcController,
      );
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(deleteAuthenticationGrpcController).toBeDefined();
  });

  it('should delete an authentication', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    await deleteAuthenticationGrpcController.delete(
      deleteAuthenticationRequest,
    );
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a dedicated RpcException if authentication does not exist', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    expect.assertions(3);
    try {
      await deleteAuthenticationGrpcController.delete(
        deleteAuthenticationRequest,
      );
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.NOT_FOUND);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a dedicated RpcException if a database error occurs', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    expect.assertions(3);
    try {
      await deleteAuthenticationGrpcController.delete(
        deleteAuthenticationRequest,
      );
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.INTERNAL);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a generic RpcException', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    expect.assertions(3);
    try {
      await deleteAuthenticationGrpcController.delete(
        deleteAuthenticationRequest,
      );
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.UNKNOWN);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });
});
